#!/bin/bash
# vim: dict=/usr/share/rhts-library/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of selinux problem reproducer
#   Description: Test basic authentication use cases.
#   Author: David Spurek <dspurek@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2012 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/bin/rhts-environment.sh
. /usr/share/rhts-library/rhtslib.sh

PACKAGES=( "nss_ldap"         \
           "openldap-clients" \
           "openldap-servers" \
           "glibc-common"     \
           "openssh"          \
           "expect"           \
           "telnet"           \
           "telnet-server"    \
           "xinetd"           \
           "authconfig"       )

# Affected authconfig files to be backed up
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#        /etc/nslcd.conf \
for file in /etc/pam.d/system-auth-ac \
        /etc/sssd \
        /etc/group \
        /etc/gshadow \
        /etc/hesiod.conf \
        /etc/krb.conf \
        /etc/krb5.conf \
        /etc/ldap.conf \
        /etc/libuser.conf \
        /etc/login.defs \
        /etc/nsswitch.conf \
        /etc/pam_smb.conf \
        /etc/passwd \
        /etc/samba/smb.conf \
        /etc/shadow \
        /etc/sysconfig/authconfig \
        /etc/sysconfig/network \
        /etc/yp.conf \
        /etc/krb5.conf \
        /var/kerberos/krb5kdc \
        /etc/krb5.keytab \
        ; do
    test -e $file && ConfigFiles="$ConfigFiles $file"
done

host=my-domain.com

function do_telnet {
expect <<EOF
spawn telnet ${host} -l $1
expect "Password:" { send "$2\r" } 
expect ".*$"
send "whoami\r"
expect "$1" { exit 0 } 

exit 2
EOF
}

function ldap_conf {
cat >/etc/openldap/ldap.conf<<EOF
URI         ldap://my-domain.com
BASE        dc=my-domain,dc=com
EOF
return $?
}

function slapd_conf {
cat >/etc/openldap/slapd.conf<<EOF
include         /etc/openldap/schema/core.schema
include         /etc/openldap/schema/cosine.schema
include         /etc/openldap/schema/inetorgperson.schema
include         /etc/openldap/schema/nis.schema

allow bind_v2

pidfile         /var/run/openldap/slapd.pid
argsfile        /var/run/openldap/slapd.args

database        bdb
suffix          "dc=my-domain,dc=com"
rootdn          "cn=admin,dc=my-domain,dc=com"
rootpw          {SSHA}GPhzu7pTYP4I+nGeujpBkODiPxX0v8n8

directory       /var/lib/ldap

index objectClass                       eq,pres
index ou,cn,mail,surname,givenname      eq,pres,sub
index uidNumber,gidNumber,loginShell    eq,pres
index uid,memberUid                     eq,pres,sub
index nisMapName,nisMapEntry            eq,pres,sub

access to attrs=shadowLastChange,userPassword
      by self write
      by * auth

access to *
      by * read

EOF
return $?
}

function nssldap_conf {
cat  >/etc/ldap.conf <<EOF
suffix                  "dc=my-domain,dc=com"
ldap_version            3
bind_timelimit          5
uri                     ldap://my-domain.com
base                    dc=my-domain,dc=com
bind_policy             soft
pam_filter              objectclass=posixAccount
pam_login_attribute     uid
pam_member_attribute    memberuid
nss_base_passwd         ou=people,dc=my-domain,dc=com
scope                   one
referrals               yes
ssl                     no
EOF
return $?
}

USERS_COUNT=1

SERVICE='ldap'

rlJournalStart

    rlPhaseStartSetup "General"

        # Check reqiured packages.
       for P in ${PACKAGES[@]}; do rlCheckRpm $P || rlDie "Package $P is missing"; done	

    rlPhaseEnd

    rlPhaseStartSetup "Configuration"

	# Backup services.
	rlServiceStop $SERVICE
	rlServiceStop "nscd"


	# Back-up settings.
	rlFileBackup "/etc/openldap/ldap.conf" 
	rlFileBackup "/etc/openldap/slapd.conf" 
	rlFileBackup "/etc/nscd.conf" 
	rlFileBackup "/etc/pam.d/"
	rlFileBackup "/var/lib/ldap/" ; rm -rf /var/lib/ldap/*
	rlFileBackup "/var/run/openldap"
	rlFileBackup "/etc/hosts"  
	rlRun "rlFileBackup $ConfigFiles" 0 "Backing up config files"

        # Setup server/client certs and configuration files.         
	rlRun "echo '127.0.0.1 my-domain.com' >> /etc/hosts" 0
    rlRun "cp -f /etc/nscd.conf . " 0 "Backup nscd.conf"      
    rlRun "ldap_conf" 0 "OpenLDAP client system configuration"

	# Setup LDAP server.
	rlRun "slapd_conf" 0 "Setting LDAP server"
	
 	# Create domain with all ldap users.
	rlRun "slapadd -l base.ldif " 0 "Creating LDAP base domain"
	rlRun "bash generator.sh $USERS_COUNT > users.ldif && \
	       slapadd -l users.ldif -q -b \"dc=my-domain,dc=com\"" 0 \
	      "Creating LDAP users"
	
	rlRun "chown -R ldap:ldap /var/lib/ldap /var/run/openldap" 0

	# Start LDAP server.
	rlRun "service $SERVICE start && sleep 10" 0 "Starting LDAP server"

        # Most commonly used configuration.
	rlRun "nssldap_conf" 0 "Configuring nss_ldap"

	# SELinux stuff, etc.
	rlRun "restorecon -Rv /etc" 0-255 "Restoring file contexts in /etc"

	rlRun "chkconfig telnet on" 0
	rlRun "service xinetd restart" 0

    rlPhaseEnd

    # Test suite.

    rlPhaseStartTest
    	rlRun "authconfig --enablemd5                   \
                              --enableldap                  \
                              --enableldapauth              \
    			  --ldapbasedn=dc=my-domain,dc=com \
    			  --enablelocauthorize          \
        			  --ldapserver=ldap://my-domain.com \
                              --disablecache                \
                              --kickstart && sleep 5" 0     \
                  "Setting clients' auth configuration via authconfig"

	rlLog "Creating files for each LDAP user"
	        
	TMP_DIR=`mktemp -d -p /tmp`
	for I in `seq -w 1 $USERS_COUNT`; do	 
	    `for J in \`seq -w 1 3\`; do touch "${TMP_DIR}/file${I}-${J}"; done`
	    chown user${I} ${TMP_DIR}/file${I}-*
	done

	rlRun "do_telnet user1 x" 0

    rlPhaseEnd

  rlPhaseStartTest "SELinux"
    rlRun "ausearch -m AVC -ts recent > /tmp/out 2>&1" 0-255
    rlRun "grep '<no matches>' /tmp/out" 0 "There should not be any SELinux AVC"
    cat /tmp/out && rm -f /tmp/out
  rlPhaseEnd

    rlPhaseStartCleanup
    
	rlRun "authconfig --enableshadow    \
                          --enablemd5       \
                          --disableldap     \
                          --disableldapauth \
	                      --disablecache    \
                          --kickstart" 0    \
         "Setting clients' auth configuration via authconfig"

    rlRun "service $SERVICE stop && sleep 10" 0 "Stopping LDAP server"

 
	rm -rf /home/ldap/
	
	rm -rf $TMP_DIR ls.out /var/run/openldap/* /var/lib/ldap/* \
	    /etc/openldap/slapd.conf
	
    rlFileRestore

	rlServiceRestore $SERVICE
	rlServiceRestore "nscd"
	
    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
